###############################################
########### VASOMATIC CHUNK ANALYZER ##########
###############################################

#  V 0.1 - 9 April 2024 - Dr Audrey Chagnot   #
#           audrey.chagnot@ed.ac.uk           #

################ [HOW TO USE] #################
#
# Run the whole script and follow instructions
# In RGui: Edit>Run all...
# In RStudio: Ctrl+Shift+Enter
#
# This script is dedicated to the analysis and
# visualization of chunk data generated by the
# VasOMatic macro in ImageJ. For reference see
# 


###### [DATA PREPARATION BEFORE IMPORT] #######
# You should save the file as a .csv filer with
# 8 headers with these names in this order:
# Subject, Group, Replicate, ROI,
# Diameter, Length, Coverage, Signal.
# Look at the ChunkData.csv provided as an
# example on the GitLab page.
# The 4 first are for ID purpose and you should
# fill these yourself as you acquire the data.
# The 4 last are output pasted from VasOMatic.
# If you don't need some of the ID headers just
# fill the column with ones.

################ [LIBRARIES] ##################

if("rlang" %in% rownames(installed.packages()) == FALSE) {install.packages("rlang")}
if("vctrs" %in% rownames(installed.packages()) == FALSE) {install.packages("vctrs")}
if("svDialogs" %in% rownames(installed.packages()) == FALSE) {install.packages("svDialogs")}
if("data.table" %in% rownames(installed.packages()) == FALSE) {install.packages("data.table")}
if("ggplot2" %in% rownames(installed.packages()) == FALSE) {install.packages("ggplot2")}
if("forcats" %in% rownames(installed.packages()) == FALSE) {install.packages("forcats")}
if("abind" %in% rownames(installed.packages()) == FALSE) {install.packages("abind")}
if("matrixStats" %in% rownames(installed.packages()) == FALSE) {install.packages("matrixStats")}
library("ggplot2")
library("forcats")
library("svDialogs")
library("data.table")
library("abind")
library("matrixStats")

########## [VARIABLES INITIALISATION] #########

CONTINUE<-TRUE
HAVE_DATASET <- FALSE

################ [CODE START] #################

while (CONTINUE == TRUE) { # start of code

############### [LOAD DATASET] ################

if (!HAVE_DATASET) {

# Recover .csv file
if (!exists("WORKPATH")) {WORKPATH = getwd()}
WORKPATH <- dlg_open(default = WORKPATH, title = "Select a .csv file")$res
DATA <- read.csv(file=WORKPATH, head=TRUE, sep=",") # Change sep if not comma (e.g. ";")
DATA <- data.frame(DATA)

DATA <- as.data.frame(DATA)

# Display Dataset content
SubjectsList <- unique(DATA[, 1])
NSub <- length(SubjectsList)
listSub <- paste(as.character(SubjectsList),collapse=", ")
GroupsList <- unique(DATA[, 2])
NGrp <- length(GroupsList)
listGrp <- paste(as.character(GroupsList),collapse=", ")
ROIsList <- unique(DATA[, 4])
NROI <- length(ROIsList)
listROI <- paste(as.character(ROIsList),collapse=", ")
message <- paste("GENERAL DATA INFORMATION", "\n\n",paste(as.character(NSub)," subject(s) detected, which are: \n",listSub), "\n\n", paste(as.character(NGrp)," group(s) detected, which are: \n",listGrp), "\n\n", paste(as.character(NROI)," ROI(s) detected, which are: \n",listROI),"\n\n", "Press OK to proceed", sep = "")
dlg_message(message, type="okcancel", gui = .GUI)

HAVE_DATASET <- TRUE
}

################# [HUB MENU] ##################

ACTIONS_LIST <- c("New Dataset","Bin Data","Scatterplots","Exit")
ACTION <- dlg_list(ACTIONS_LIST, multiple = FALSE, title = "What do you want to do?")$res

########### [ACTION 1: NEW DATASET] ###########

if (ACTION == "New Dataset") {HAVE_DATASET <- FALSE} # Will force to load a dataset at the next iteration

############ [ACTION 2: BIN DATA] ###########

if (ACTION == "Bin Data") {

# Specify range and bin size - TODO - Embed in GUI
XMIN <- 1
XMAX <- 10
YMIN <- 0
YMAX <- 100
BinSize <- 0.5

# Create Bins for plotting and BinsID for the labels
Bins <- seq(XMIN,XMAX,by=BinSize)
Bins <- c(0,Bins,1000)
BinsID <- array(0,length(Bins)-1)
for (b in 1:(length(Bins)-1)) {
  if (b==1) {BinsID[b] <- paste0("<",as.character(Bins[2]))}
  else if (b==length(Bins)-1) {BinsID[b] <- paste0(">",as.character(Bins[b]))}
  else {BinsID[b] <- paste0(as.character(Bins[b]),"-",as.character(Bins[b+1]))}
}
NBins <- length(Bins)

# Selection of the ROI
SELECTEDROI <- dlg_list(ROIsList, multiple = FALSE, title = "Chose ROI to plot")$res

# Creating bins
BinsMeans <- array(0,c((NBins-1),length(SubjectsList)))
BinsLengths <- array(0,c((NBins-1),length(SubjectsList)))
SubjGroups <- array(0,length(SubjectsList))
for (s in 1:length(SubjectsList)) {
  SGroup <- DATA[DATA[, 1] == SubjectsList[s],2]
  for (b in 1:(NBins-1)) {
    CovVals <- DATA[DATA[, 1] == SubjectsList[s] & DATA[, 4] == SELECTEDROI & DATA[, 5] > Bins[b] & DATA[, 5] <= Bins[b+1],7]
    BinsMeans[b,s] <- mean(CovVals)
    BinsLengths[b,s] <- length(CovVals)
  }
  SubjGroups[s] <- SGroup[1]
}

# Allocating matrices for storing grouped data
BinsMeansPerGroup <- array(numeric(),c(NBins-1,length(SubjectsList),0)) 
for (g in 1:length(GroupsList)) {
BinsMeansPerGroup <- abind(BinsMeansPerGroup, matrix(NaN,nrow=NBins-1,ncol=length(SubjectsList)), along=3)}
Means <- array(1,0)
SDs <- array(1,0)
Group <- array(1,0)
NBVals <- array(1,0)
for (s in 1:length(SubjectsList)) {
Group <- which(SubjGroups[s] == GroupsList)[[1]]
BinsMeansPerGroup[,s,Group] <- BinsMeans[,s]}
Group <- array(1,0)
for (g in 1:length(GroupsList)) {
Means <- c(Means,rowMeans(BinsMeansPerGroup[,,g], na.rm = TRUE))
SDs <- c(SDs,rowSds(BinsMeansPerGroup[,,g], na.rm = TRUE))
Group <- c(Group,rep(GroupsList[g],NBins-1))
NBVals <- c(NBVals,rowSums(!is.nan(BinsMeansPerGroup[,,g])))}
Means[is.nan(Means)] <- NA
SDs[is.nan(SDs)] <- NA

IC95Lo <- Means - 1.96*(SDs/sqrt(NBVals))
IC95Hi <- Means + 1.96*(SDs/sqrt(NBVals))
ForceOrder <- (1:length(BinsID)) # Otherwise will disorganize
BinFrame<-data.frame(rep(BinsID,length(GroupsList)),Means,Group,IC95Lo,IC95Hi,ForceOrder)

grp = as.character(BinFrame[,3])
HISTOGRAM <- ggplot(BinFrame, aes(x=fct_reorder(BinFrame[,1],BinFrame[,6]) , y=BinFrame[,2], group=grp, color=grp)) + 
  geom_ribbon(aes(ymin=BinFrame[,4], ymax=BinFrame[,5], fill=grp), alpha=0.1, color=NA) +  
  geom_line() +
  geom_point() +
  xlab("Local Vessel Diameter (um)") +
  scale_y_continuous(name = "Local Pericyte Coverage (%)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  coord_cartesian(xlim = c((XMIN*2)-XMIN/2, (XMAX*2)-XMIN/2), ylim = c(YMIN, YMAX)) +
  labs(title = SELECTEDROI) +
  theme(panel.background = element_rect(fill = 'white', colour = 'black'),
  legend.position="none",
  panel.grid.major = element_line(colour="light gray"),
  panel.grid.minor = element_line(colour="light gray"),
  legend.key = element_blank())
print(HISTOGRAM)

# Save plot if user is satisfied
res <- dlg_message("Save graph?", "yesno")$res
if (res == "yes") {
HistName = paste(SELECTEDROI,'histogram.tiff',sep="_")
tiff(paste(dirname(WORKPATH),HistName,sep="/"), units="cm", width=10, height=10, res=300)
print(HISTOGRAM)
dev.off()
dlg_message(paste("Histogram succesfully saved as",HistName,"in",dirname(WORKPATH)))$res
}

# Save data for further statistics
res <- dlg_message("Save binned data as .csv?", "yesno")$res
if (res == "yes") {
BinsMeansPerGroup[is.nan(BinsMeansPerGroup)] <- 0
ExportTable <- BinsMeansPerGroup[,,1] + BinsMeansPerGroup[,,2]
ExportTable[ExportTable==0] <- NA
colnames(ExportTable) <- SubjectsList
rownames(ExportTable) <- BinsID
write.csv(ExportTable,paste(dirname(WORKPATH),paste0(SELECTEDROI,"_BinnedData.csv"),sep="/"),row.names=TRUE)
dlg_message(paste("Binned data succesfully saved as",paste0(SELECTEDROI,"_BinnedData.csv"),"in",dirname(WORKPATH)))$res
}

} # End of HISTOGRAMS ACTION

########## [ACTION 3: SCATTERPLOTS] ###########

if (ACTION == "Scatterplots") {

# Chose representation type
SCAT_ACTIONS_LIST <- c("Group vs Group","Single ROI of Subject","ROI vs ROI","Exit")
SCAT_ACTION <- dlg_list(SCAT_ACTIONS_LIST, multiple = FALSE, title = "Do Scatterplots For What?")$res

# Select plotting range, point and line size here. - TODO - embed in GUI
XMIN <- 1
XMAX <- 10
YMIN <- 0
YMAX <- 100
COLOR <- 'blue'
MakePoints <- FALSE

##### [SCATTERPLOT 1: GROUP vs GROUP]

if (SCAT_ACTION == "Group vs Group") {

# Selection of the ROI
SELECTEDROI <- dlg_list(ROIsList, multiple = FALSE, title = "Chose ROI for Group-wise plot")$res

# Grab data in the main folder
Data = data.frame() # Initialize
for (g in 1:length(GroupsList)) {
Diameters <- DATA[DATA[, 2] == GroupsList[g] & DATA[, 4] == SELECTEDROI,5]
Coverages <- DATA[DATA[, 2] == GroupsList[g] & DATA[, 4] == SELECTEDROI,7]
Data <- rbind(Data, data.frame(Diameters, Coverages, group=as.character(GroupsList[g])))
} # End of data recovery loop
Data <- Data[Data[, 1] >= XMIN & Data[, 1] <= XMAX & Data[, 2] >= YMIN & Data[, 2] <= YMAX,]

# Draw the plot
SCATTERPLOT <-
  ggplot(Data, aes(x=Diameters,y=Coverages)) +
  geom_density_2d(size = 2, aes(color = group, alpha=..level..)) +
  scale_x_continuous(name = "Local Vessel Diameter (um)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  scale_y_continuous(name = "Local Pericyte Coverage (%)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  coord_cartesian(xlim = c(XMIN, XMAX), ylim = c(YMIN, YMAX)) +
  labs(title = SELECTEDROI) +
  theme(panel.background = element_rect(fill = 'white', colour = 'black'),
  #legend.position="none",
  panel.grid.major = element_line(colour="light gray"),
  panel.grid.minor = element_line(colour="light gray"),
  legend.key = element_blank())
if (MakePoints == TRUE) {SCATTERPLOT <- SCATTERPLOT + geom_point(size = 1)}
print(SCATTERPLOT)

# Save plot if user is satisfied
res <- dlg_message("Save graph?", "yesno")$res
if (res == "yes") {
PlotName = paste(SELECTEDROI,'Grouped.tiff',sep="_")
tiff(paste(dirname(WORKPATH),PlotName,sep="/"), units="cm", width=10, height=10, res=300)
print(SCATTERPLOT)
dev.off()
dlg_message(paste("Plot succesfully saved as",PlotName,"in",dirname(WORKPATH)))$res
}

} # End of SCATTERPLOT 1


##### [SCATTERPLOT 2: SINGLE ROI of SUBJECT]

if (SCAT_ACTION == "Single ROI of Subject") {

# Selection of the subject and ROI
SUBJECTROI_LIST <- do.call(paste, c(expand.grid(ROIsList,SubjectsList), list(sep=';')))
SELECTED_SUBJECTROI <- dlg_list(SUBJECTROI_LIST, multiple = FALSE, title = "Chose Subject/ROI")$res
SELECTED_SUBJECTROI <- strsplit(SELECTED_SUBJECTROI,';')
SELECTED_SUBJECTROI <- SELECTED_SUBJECTROI[[1]]
SELECTEDROI <- SELECTED_SUBJECTROI[1]
SELECTEDSUBJECT <- SELECTED_SUBJECTROI[2]

# Grab data in the main folder
Diameters <- DATA[DATA[, 1] == SELECTEDSUBJECT & DATA[, 4] == SELECTEDROI,5]
Coverages <- DATA[DATA[, 1] == SELECTEDSUBJECT & DATA[, 4] == SELECTEDROI,7]
Data <- data.frame(Diameters,Coverages)
Data <- Data[Data[, 1] >= XMIN & Data[, 1] <= XMAX & Data[, 2] >= YMIN & Data[, 2] <= YMAX,]

# Draw the plot
SCATTERPLOT <-
  ggplot(Data, aes(x=Diameters,y=Coverages)) +
  geom_density_2d(size = 2, aes(alpha=..level..)) +
  scale_x_continuous(name = "Local Vessel Diameter (um)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  scale_y_continuous(name = "Local Pericyte Coverage (%)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  coord_cartesian(xlim = c(XMIN, XMAX), ylim = c(YMIN, YMAX)) +
  labs(title = paste(SELECTEDROI,'of subject',SELECTEDSUBJECT)) +
  theme(panel.background = element_rect(fill = 'white', colour = 'black'),
  legend.position="none",
  panel.grid.major = element_line(colour="light gray"),
  panel.grid.minor = element_line(colour="light gray"),
  legend.key = element_blank())
if (MakePoints == TRUE) {SCATTERPLOT <- SCATTERPLOT + geom_point(size = 1)}
print(SCATTERPLOT)

# Save plot if user is satisfied
res <- dlg_message("Save graph?", "yesno")$res
if (res == "yes") {
PlotName = paste(SELECTEDSUBJECT,SELECTEDROI,'.tiff',sep="_")
tiff(paste(dirname(WORKPATH),PlotName,sep="/"), units="cm", width=10, height=10, res=300)
print(SCATTERPLOT)
dev.off()
dlg_message(paste("Plot succesfully saved as",PlotName,"in",dirname(WORKPATH)))$res
}

} # End of SCATTERPLOT 2

##### [SCATTERPLOT 3: ROI vs ROI]

if (SCAT_ACTION == "ROI vs ROI") {

# Selection of the GROUP
SELECTEDGROUP <- dlg_list(GroupsList, multiple = FALSE, title = "Chose Group for ROI-wise plot")$res

# Grab data in the main folder

Data = data.frame() # Initialize
for (r in 1:length(ROIsList)) {
Diameters <- DATA[DATA[, 2] == SELECTEDGROUP & DATA[, 4] == ROIsList[r],5]
Coverages <- DATA[DATA[, 2] == SELECTEDGROUP & DATA[, 4] == ROIsList[r],7]
Data <- rbind(Data, data.frame(Diameters, Coverages, group=as.character(ROIsList[r])))
} # End of data recovery loop
Data <- Data[Data[, 1] >= XMIN & Data[, 1] <= XMAX & Data[, 2] >= YMIN & Data[, 2] <= YMAX,]

# Draw the plot
SCATTERPLOT <-
  ggplot(Data, aes(x=Diameters,y=Coverages)) +
  geom_density_2d(size = 2, aes(color = group, alpha=..level..)) +
  scale_x_continuous(name = "Local Vessel Diameter (um)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  scale_y_continuous(name = "Local Pericyte Coverage (%)", breaks = scales::pretty_breaks(n=10), expand = c(0, 0)) +
  coord_cartesian(xlim = c(XMIN, XMAX), ylim = c(YMIN, YMAX)) +
  labs(title = paste('Group',SELECTEDGROUP,'all ROIs')) +
  theme(panel.background = element_rect(fill = 'white', colour = 'black'),
  #legend.position="none",
  panel.grid.major = element_line(colour="light gray"),
  panel.grid.minor = element_line(colour="light gray"),
  legend.key = element_blank())
if (MakePoints == TRUE) {SCATTERPLOT <- SCATTERPLOT + geom_point(size = 1)}
print(SCATTERPLOT)

# Save plot if user is satisfied
res <- dlg_message("Save graph?", "yesno")$res
if (res == "yes") {
PlotName = paste("Group",SELECTEDGROUP,"AllROIsPlot.tiff",sep="_")
tiff(paste(dirname(WORKPATH),PlotName,sep="/"), units="cm", width=10, height=10, res=300)
print(SCATTERPLOT)
dev.off()
dlg_message(paste("Plot succesfully saved as",PlotName,"in",dirname(WORKPATH)))$res
}

} # End of SCATTERPLOT 3

} # End of SCATTERPLOT ACTION

############## [ACTION 4: EXIT] ###############

if (ACTION == "Exit") {CONTINUE<-FALSE}

################## [CODE END] #################

} # end of code